atStatus_t AT_Advertise(uint8_t *addr, uint8_t addrType, uint8_t dutyCycle)
{
  static atStatus_t status = AT_STATUS_OK;

  ble_gap_addr_t        peer_addr;
  ble_gap_adv_params_t *adv_params = &at_state()->adv_params;

  memset(adv_params, 0, sizeof(ble_gap_adv_params_t));

  advSettings_t *adv = &at_config()->adv;
  atState_t *state = at_state();

  if (!(adv->config.channelMap & ATSDSLE_AD_CH_37))
  {
    adv_params->channel_mask[4] |= 0x20;
  }

  if (!(adv->config.channelMap & ATSDSLE_AD_CH_38))
  {
    adv_params->channel_mask[4] |= 0x40;
  }

  if (!(adv->config.channelMap & ATSDSLE_AD_CH_39))
  {
    adv_params->channel_mask[4] |= 0x80;
  }

  if (addr != NULL)
  {
    memcpy(peer_addr.addr, addr, BLE_GAP_ADDR_LEN);
    peer_addr.addr_type = at_utils_translate_to_nrf_addr_type(addrType);

    adv_params->p_peer_addr = &peer_addr;
  }
  else
  {
    adv_params->p_peer_addr = NULL;
  }

  // If extended
  if (adv->config.adType >= ATSDSLE_AD_TYPE_EXT_CONN)
  {
    switch (adv->config.adType)
    {
      case ATSDSLE_AD_TYPE_EXT_CONN:
        {
          adv_params->properties.type = BLE_GAP_ADV_TYPE_EXTENDED_CONNECTABLE_NONSCANNABLE_UNDIRECTED;

          state->adv_scan_data.adv_data.p_data = state->adv_ext_data.p_data;
          state->adv_scan_data.adv_data.len    = state->adv_ext_data.len;

          state->adv_scan_data.scan_rsp_data.p_data = 0;
          state->adv_scan_data.scan_rsp_data.len    = 0;
        }
        break;

      case ATSDSLE_AD_TYPE_EXT_SCAN:
        {
          adv_params->properties.type = BLE_GAP_ADV_TYPE_EXTENDED_NONCONNECTABLE_SCANNABLE_UNDIRECTED;

          state->adv_scan_data.adv_data.p_data = 0;
          state->adv_scan_data.adv_data.len    = 0;

          state->adv_scan_data.scan_rsp_data.p_data = &state->adv_ext_data.p_data[3];  // Skip flags since scan response cant have flags
          state->adv_scan_data.scan_rsp_data.len    = state->adv_ext_data.len - 3;
        }
        break;

      case ATSDSLE_AD_TYPE_EXT_NON_CONN_SCAN:
        {
          adv_params->properties.type = BLE_GAP_ADV_TYPE_EXTENDED_NONCONNECTABLE_NONSCANNABLE_UNDIRECTED;

          state->adv_scan_data.adv_data.p_data = state->adv_ext_data.p_data;
          state->adv_scan_data.adv_data.len    = state->adv_ext_data.len;

          state->adv_scan_data.scan_rsp_data.p_data = 0;
          state->adv_scan_data.scan_rsp_data.len    = 0;
        }
        break;

      default:
        break;
    }

    if (addr != NULL)
    {
      // Directed type is 1 more than the undirected
      adv_params->properties.type++;
    }

    adv_params->properties.include_tx_power = 0; // Not currently supported by softdevice
    adv_params->primary_phy   = adv->config.primaryPhy;
    adv_params->secondary_phy = adv->config.secondaryPhy;
    adv_params->properties.anonymous = adv->config.anonymous;

    at_state()->slaveConnectPhy = adv_params->secondary_phy;
  }
  else
  {
    if (addr != NULL)
    {
      state->adv_scan_data.adv_data.p_data  = 0;
      state->adv_scan_data.adv_data.len     = 0;

      state->adv_scan_data.scan_rsp_data.p_data = 0;
      state->adv_scan_data.scan_rsp_data.len    = 0;

      if (dutyCycle == ATSDSDLE_DUTY_CYCLE_HIGH)
      {
        adv_params->properties.type = BLE_GAP_ADV_TYPE_CONNECTABLE_NONSCANNABLE_DIRECTED_HIGH_DUTY_CYCLE;
      }
      else
      {
        adv_params->properties.type = BLE_GAP_ADV_TYPE_CONNECTABLE_NONSCANNABLE_DIRECTED;
      }
    }
    else
    {
      switch (adv->config.adType)
      {
        case ATSDSLE_AD_TYPE_CONN_SCAN:
          {
            adv_params->properties.type = BLE_GAP_ADV_TYPE_CONNECTABLE_SCANNABLE_UNDIRECTED;

            state->adv_scan_data.adv_data.p_data  = state->adv_data.p_data;
            state->adv_scan_data.adv_data.len     = state->adv_data.len;

            state->adv_scan_data.scan_rsp_data.p_data = state->scan_rsp_data.p_data;
            state->adv_scan_data.scan_rsp_data.len    = state->scan_rsp_data.len;
          }
          break;

        case ATSDSLE_AD_TYPE_SCAN:
          {
            adv_params->properties.type = BLE_GAP_ADV_TYPE_NONCONNECTABLE_SCANNABLE_UNDIRECTED;

            state->adv_scan_data.adv_data.p_data  = state->adv_data.p_data;
            state->adv_scan_data.adv_data.len     = state->adv_data.len;

            state->adv_scan_data.scan_rsp_data.p_data = state->scan_rsp_data.p_data;
            state->adv_scan_data.scan_rsp_data.len    = state->scan_rsp_data.len;
          }
          break;

        case ATSDSLE_AD_TYPE_NON_CONN_SCAN:
          {
            adv_params->properties.type = BLE_GAP_ADV_TYPE_NONCONNECTABLE_NONSCANNABLE_UNDIRECTED;

            state->adv_scan_data.adv_data.p_data  = state->adv_data.p_data;
            state->adv_scan_data.adv_data.len     = state->adv_data.len;

            state->adv_scan_data.scan_rsp_data.p_data = 0;
            state->adv_scan_data.scan_rsp_data.len    = 0;
          }
          break;

        default:
          break;
      }
    }

    adv_params->primary_phy  = BLE_GAP_PHY_1MBPS;
    at_state()->slaveConnectPhy = BLE_GAP_PHY_1MBPS;
  }

  //TODO - use all adv params
//adv_params->max_adv_evts = 10; // TODO - maybe use as a param for ATDSLE?
  adv_params->filter_policy = adv->config.whiteListFilter;

  if (at_state_le()->connected)
  {
    adv_params->interval = adv->timing.connectedInt;
  }
  else
  {
    adv_params->interval = adv->timing.unconnectedInt;
  }

  adv_params->duration = adv->timing.timeout * 100;

  if (at_state_le()->advertising)
  {
    at_cancel(AT_CMD_TYPE_LE, AT_CMD_LE_ATDSLE, 0, 1);
  }

  bool setAdPower = false;

  if (at_state()->adv_handle == BLE_GAP_ADV_SET_HANDLE_NOT_SET)
  {
    setAdPower = true;
  }

  status = sd_ble_gap_adv_set_configure(&at_state()->adv_handle, &state->adv_scan_data, &state->adv_params);

  NRF_LOG_DEBUG("STATUS=%u", status);

  if (setAdPower)
  {
    uint32_t err;

    err = sd_ble_gap_tx_power_set(BLE_GAP_TX_POWER_ROLE_ADV, 0, at_config()->power.adv);

    NRF_LOG_DEBUG("Ad power set! err=%u", err);
  }

  if (status == AT_STATUS_OK)
  {
    status = sd_ble_gap_adv_start(at_state()->adv_handle, AT_BLE_CONN_CFG_TAG);

    // If error was due to already being connected as a peripheral then change the ad type to nonconnectable
    if (status == NRF_ERROR_CONN_COUNT)
    {
      if (adv->config.adType >= ATSDSLE_AD_TYPE_EXT_CONN)
      {
        adv_params->properties.type = BLE_GAP_ADV_TYPE_EXTENDED_NONCONNECTABLE_NONSCANNABLE_UNDIRECTED;

        sd_ble_gap_adv_set_configure(&at_state()->adv_handle, &state->adv_scan_data, &state->adv_params);
      }
      else
      {
        adv_params->properties.type = BLE_GAP_ADV_TYPE_NONCONNECTABLE_NONSCANNABLE_UNDIRECTED;

        sd_ble_gap_adv_set_configure(&at_state()->adv_handle, &state->adv_scan_data, &state->adv_params);
      }

      status = sd_ble_gap_adv_start(at_state()->adv_handle, AT_BLE_CONN_CFG_TAG);

//    at_state()->autoAdNonConnectable = 1;
    }

    status = convertNrfError(status, false);

    if (status == AT_STATUS_OK)
    {
      at_state_update(LE_STATE_ADVERTISING, 1, 0);
    }
  }

  return status;
}

void load_lists(uint8_t init)
{
  uint8_t      i = 0;
  uint8_t      wlCount;
  ret_code_t   ret;
  uint32_t     peer_cnt;
  pm_peer_id_t peer_id;
  pm_peer_id_t peers[AT_MAX_PAIRED];
  pm_peer_data_bonding_t pm_peer_data_bonding;

  memset(peers, PM_PEER_ID_INVALID, sizeof(peers));

  // Load all peers from flash
  peer_id = pm_next_peer_id_get(PM_PEER_ID_INVALID);

  NRF_LOG_DEBUG("sizeof(pm_peer_data_bonding_t)=%u", sizeof(pm_peer_data_bonding_t));

  while (peer_id != PM_PEER_ID_INVALID)
  {
    peers[i] = peer_id;

    pm_peer_data_bonding_load(peer_id, &pm_peer_data_bonding);

    NRF_LOG_DEBUG("peer_id = %u, addrType=%u peer=%u", peer_id,
                  pm_peer_data_bonding.peer_ble_id.id_addr_info.addr_type,
                  pm_peer_data_bonding.peer_ble_id.id_addr_info.addr_id_peer);

//  NRF_LOG_DEBUG("%s", AT_AddrToStrWithType(pm_peer_data_bonding.peer_ble_id.id_addr_info.addr, pm_peer_data_bonding.peer_ble_id.id_addr_info.addr_type));

    // Copy to local state list
    at_state()->pairedList.devs[i].addrType = pm_peer_data_bonding.peer_ble_id.id_addr_info.addr_type;
    memcpy(at_state()->pairedList.devs[i].publicAddr, pm_peer_data_bonding.peer_ble_id.id_addr_info.addr, BLE_GAP_ADDR_LEN);

    i++;

    peer_id = pm_next_peer_id_get(peer_id);
  }

  at_state()->pairedList.count = peer_cnt = i;

  if (peer_cnt >= AT_MAX_PAIRED)
  {
    if (!at_state()->maxPaired)
    {
      secConfig_t *secConfig = &at_config()->secConfig;

      at_state()->maxPaired = 1;

      AT_SetPairing(secConfig->pairingMode, secConfig->mitm, secConfig->ioCapabilities, secConfig->syncWl, secConfig->autoDisconn, secConfig->lesc);
    }
  }
  else
  {
    if (at_state()->maxPaired)
    {
      secConfig_t *secConfig = &at_config()->secConfig;

      at_state()->maxPaired = 0;

      AT_SetPairing(secConfig->pairingMode, secConfig->mitm, secConfig->ioCapabilities, secConfig->syncWl, secConfig->autoDisconn, secConfig->lesc);
    }
  }

  wlCount = (peer_cnt > AT_MAX_WHITE_LIST) ? AT_MAX_WHITE_LIST : peer_cnt;

  // If syncing whitelist with pair list
  if (at_config()->secConfig.syncWl)
  {
    ret = pm_whitelist_set(peers, wlCount);
    NRF_LOG_DEBUG("sd_ble_gap_whitelist_set=0x%02X, peer_cnt=%u", ret, wlCount);
  }
  else if (init)
  {
    atWhiteList_t whiteList;

    AT_GetWhiteList(&whiteList);

    if (whiteList.count)
    {
      ble_gap_addr_t const *addr_ptrs[AT_MAX_WHITE_LIST];

      for (uint32_t i = 0; i < whiteList.count; i++)
      {
        addr_ptrs[i] = (ble_gap_addr_t*)&whiteList.devs[i];
        whiteList.devs[i].addrType = at_utils_translate_to_nrf_addr_type(whiteList.devs[i].addrType);
      }

      ret = sd_ble_gap_whitelist_set(addr_ptrs, whiteList.count);
      NRF_LOG_DEBUG("sd_ble_gap_whitelist_set=0x%02X, peer_cnt=%u", ret, whiteList.count);
    }
  }

//BLE_GAP_DEVICE_IDENTITIES_MAX_COUNT
  ret = pm_device_identities_list_set(peers, wlCount);
  NRF_LOG_DEBUG("pm_device_identities_list_set=0x%02X, peer_cnt=%u", ret, wlCount);
}
